package fxReseptit;

import javafx.scene.control.TextField;
import javafx.scene.control.TextArea;
import java.util.List;

import fi.jyu.mit.fxgui.ComboBoxChooser;
import fi.jyu.mit.fxgui.Dialogs;
import fi.jyu.mit.fxgui.ListChooser;
import fi.jyu.mit.fxgui.ModalController;
import fi.jyu.mit.fxgui.ModalControllerInterface;
import javafx.fxml.FXML;
import javafx.scene.control.Button;
import javafx.stage.Stage;
import reseptit.Ainesosa;
import reseptit.Ohje;
import reseptit.Reseptit;
import reseptit.SailoException;


/**
 * Ikkunan, jossa muokataan reseptejä, toiminta
 * @author Omistaja
 * @version 18.2.2025
 *
 */
public class OhjeetGUIController implements ModalControllerInterface<Ohje>{

    @FXML private Button nLisaaAinesosa;

    @FXML private Button nPoistaAinesosa;

    @FXML private Button nReseptiPeruuta;

    @FXML private Button nTallenna;
    
    @FXML private ListChooser<Ainesosa> ainesosatLuettelo;
    
    @FXML private TextField OhjeenKoko;

    @FXML private TextField OhjeenNimi;
    
    @FXML private ComboBoxChooser<String> Kesto;

    @FXML private TextArea OhjeenTeksti;

    
    private Reseptit reseptit;
    private Ohje oletus;
    private Ohje uusi;
    
    
    /**
     * Avaa ikkunan jossa voidaan lisätä ainesosia
     */
    @FXML void AvaaAinesosanLisays() {
        Ainesosa Uusi = ModalController.showModal(OhjeetGUIController.class.getResource("AinesosaGUIView.fxml"), "Aineosa", null, new Ainesosa(uusi.getId()));
        if (Uusi == null) return; 
        reseptit.lisaaA(Uusi);
        try {
            reseptit.talleta();
        } catch (SailoException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
        paivita();
        
    }

    
    /**
     * Palaa takaisin pääikkunaan
     */
    @FXML void PeruutaReseptinMuokkaus() {
        ModalController.closeStage(nTallenna);
    }

    @FXML void PoistaValittuAinesosa() {
        //Dialogs.showMessageDialog("Vielä ei osata poistaa ainesosia.");
        poistaAinesosa();
    }

    @FXML void Tallenna() throws SailoException {
        
        if (oletus != null) {
            uusi.setNimi(OhjeenNimi.getText());
            asetaKoko(uusi);
            uusi.setKesto(Kesto.getSelectedText());
            uusi.setTeksti(OhjeenTeksti.getText());
            reseptit.lisaa(uusi);
            reseptit.poistaVainOhje(oletus);
            reseptit.talleta();
            ModalController.closeStage(nTallenna);
        }
        else {
            
            uusi.setNimi(OhjeenNimi.getText());
            asetaKoko(uusi);
            uusi.setKesto(Kesto.getSelectedText());
            uusi.setTeksti(OhjeenTeksti.getText());
            reseptit.lisaa(uusi);
            reseptit.talleta();
            ModalController.closeStage(nTallenna);
        }
    }
    
    
    /**
     * Luodaan ohjeenkysymisdialogi ja palautetaan muokattu tai lisätty ohje
     * @param modalityStage mille ollaan modaalisia, null = sovellukselle
     * @param oletus miten ohje näytetään oletuksena
     * @param reseptit reseptiti
     * @return null jos painetaan Peruuta, muuten muokattu ohje
     */
    public static Ohje kysyOhje(Stage modalityStage, Ohje oletus, Reseptit reseptit) {
        
        return ModalController.<Ohje, OhjeetGUIController>showModal(
                OhjeetGUIController.class.getResource("OhjeetGUIView.fxml"),
                "Ohjeet",
                modalityStage, oletus, ctrl -> ctrl.setReseptit(reseptit));
    }
    
    
    /**
     * 
     */
    private void paivita() {
        ainesosatLuettelo.clear();
        List<Ainesosa> ainesosat = reseptit.annaAinesosat(uusi);
        for (Ainesosa ain : ainesosat) {
            String teksti = reseptit.getAnimi(ain) + " " + reseptit.getAmaara(ain) + " " + reseptit.getAyksikko(ain);
            ainesosatLuettelo.add(teksti, ain);
        }
    }
    
    
    /**
     * Jakaa koko-tekstikenttään syötetyn tiedon kahdelle eri muuttujalle
     * @param ohje ohje jonka tietoja tämä muuttaa
     * @return palauttaa onko muutos laillinen
     */
    private boolean asetaKoko(Ohje ohje) {
        String koko = OhjeenKoko.getText();
        String[] osat = koko.split(" ");
        if (osat.length != 2) {
            Dialogs.showMessageDialog("Väärä muoto ohjeen koolle");
            return false;
        }
        ohje.setAnnosKoko(Integer.parseInt(osat[0]));
        ohje.setAnnosTyyppi(osat[1]);
        return true;
    }
    
    
    


    /**
     * Määrittää reseptit
     * @param reseptit reseptit
     */
    private void setReseptit(Reseptit reseptit) {
        this.reseptit = reseptit;
    }
    
    
    /**
     * Poistetaan ainesosalistasta valitusta kohdasta ainesosan. 
     */
    private void poistaAinesosa() {
        int rivi = ainesosatLuettelo.getSelectionModel().getSelectedIndex();
        if ( rivi < 0 ) return;
        Ainesosa ainesosa = ainesosatLuettelo.getSelectedObject();
        if ( ainesosa == null ) return;
        reseptit.poistaAinesosa(ainesosa);
        int ainesosia = ainesosatLuettelo.getItems().size(); 
        if ( rivi >= ainesosia ) rivi = ainesosia -1;
        ainesosatLuettelo.getFocusModel().focus(rivi);
        ainesosatLuettelo.getSelectionModel().select(rivi);
        paivita();
    }


    @Override
    public Ohje getResult() {
        // TODO Auto-generated method stub
        return null;
    }


    @Override
    public void handleShown() {
        
        if (oletus != null) {
          
        OhjeenNimi.setText(reseptit.getNimi(oletus));
        OhjeenKoko.setText(reseptit.getAnnoskoko(oletus) + " " + reseptit.getAnnostyyppi(oletus));
        OhjeenTeksti.setText(reseptit.getTeksti(oletus));
        List<Ainesosa> ainesosat = reseptit.annaAinesosat(oletus);
        for (Ainesosa ain : ainesosat) {
            String teksti = reseptit.getAnimi(ain) + " " + reseptit.getAmaara(ain) + " " + reseptit.getAyksikko(ain);
            ainesosatLuettelo.add(teksti, ain);
        }
        } else {
            OhjeenNimi.setText(reseptit.getNimi(uusi));
            OhjeenKoko.setText(reseptit.getAnnoskoko(uusi) + " " + reseptit.getAnnostyyppi(uusi));
            OhjeenTeksti.setText(reseptit.getTeksti(uusi));
            List<Ainesosa> ainesosat = reseptit.annaAinesosat(uusi);
            for (Ainesosa ain : ainesosat) {
                String teksti = reseptit.getAnimi(ain) + " " + reseptit.getAmaara(ain) + " " + reseptit.getAyksikko(ain);
                ainesosatLuettelo.add(teksti, ain);
            }
        }
    }


    @Override
    public void setDefault(Ohje arg0) {
        this.oletus = arg0;
        if (arg0 == null) {
            this.uusi = new Ohje();
        } else
            try {
                this.uusi = oletus.clone();
            } catch (CloneNotSupportedException e) {
                // TODO Auto-generated catch block
                e.printStackTrace();
            }
    }

}