package fxReseptit;

import javafx.fxml.FXML;
import javafx.scene.control.Button;
import javafx.scene.control.MenuItem;
import javafx.scene.control.TextField;
import reseptit.Ainesosa;
import reseptit.Ohje;
import reseptit.SailoException;
import reseptit.Reseptit;
import fi.jyu.mit.fxgui.*;
import javafx.application.Platform;


import javafx.scene.control.TextArea;

import java.util.Collection;
import java.util.List;
import java.io.PrintStream;
import fi.jyu.mit.fxgui.ListChooser;


/**
 * @author Omistaja
 * @version 18.2.2025
 * Pääohjelman eli pääikkunan kontrolleri
 */
public class ReseptitGUIController implements ModalControllerInterface<String>{

    @FXML private MenuItem Apua;

    @FXML private MenuItem Muokkaa;

    @FXML private MenuItem Poista;

    @FXML private MenuItem Sulje;

    @FXML private MenuItem Tiedot;

    @FXML private MenuItem Tulosta;

    @FXML private MenuItem lisaaResepti;
    
    @FXML private ListChooser<Ohje> ReseptiLista;

    @FXML private Button nMuokkaa;

    @FXML private TextField ohjehaku;
    
    @FXML private TextArea TietoKentta;

    @FXML private ListChooser<Ainesosa> chooserOhjeet;
    
    @FXML private TextArea OhjeArea;
    
    @FXML private ComboBoxChooser<String> Lajittelu;

    
    /**
     * Avaa ikkunan jossa voidaan muokata reseptiä
     */
    @FXML void AvaaReseptinMuokkaus() {
        OhjeetGUIController.kysyOhje(null, ohjeKohdalla, reseptit);
        //if (ohje != null) reseptit.korvaa(ohje);
        paivita();
    }

    @FXML void AvaaReseptinMuokkausTyhjana() {
        OhjeetGUIController.kysyOhje(null, null, reseptit);
        paivita();
    }

    
    /**
     * Avaa ikkunan josta nähdään ohjelman tietoja
     */
    @FXML void AvaaTiedot() {
        ModalController.showModal(TiedotGUIController.class.getResource("TiedotGUIView.fxml"), "Tiedot", null, "");
    }

    @FXML void PoistaResepti() {
        poistaOhje();
    }
    
    
    /**
     * Hakee hakupalkissa olevan tekstin perusteella ohjetta
     */
    @FXML void HaeOhjetta() {
        if ( ohjehaku != null )
           tayta(0); 
    }
    
    @FXML void Sammutus() {
        Platform.exit();
    }

    @FXML void TulostaResepti() {
        Dialogs.showMessageDialog("Vielä ei osata tulostaa reseptiä.");
    }
    
    @FXML void NaytaOhjeenTiedot() {
        naytaTiedot();
    }

    
    private Reseptit reseptit;
    private Ohje ohjeKohdalla;
    
    
    /**
     * @param reseptit mitkä avataan listaan
     */
    public void setReseptit(Reseptit reseptit) {
        
        this.reseptit = reseptit;
        ReseptiLista.addSelectionListener(e -> naytaTiedot());
        try {reseptit.lueTiedostosta("reseptit");
        } catch (SailoException ex) {
           Dialogs.showMessageDialog(ex.getMessage()); 
        }
        tayta(0);
    }
    
    
    /**
     * Päivittää ohjelistan
     */
    public void paivita() {
        ReseptiLista.clear();
        for (int i = 0; i<reseptit.getLkm(); i++){
            Ohje s = reseptit.annaOhje(i);
            ReseptiLista.add(s.getNimi(), s);
        }
    }
    
    
    private String tallenna() {
        try {
            reseptit.talleta();
            return null;
        } catch (SailoException ex) {
            Dialogs.showMessageDialog("Tallennuksessa ongelmia! " + ex.getMessage());
            return ex.getMessage();
        }
    }
   
    
    /**
     *  Lisää reseptilistaan uusia ohjeita
     */
    public void Lisaa() {
        Ohje malli = new Ohje();
        malli.Malliohje();
        reseptit.lisaa(malli);
        paivita();
        ReseptiLista.setSelectedIndex(0);
    }
    
    
    /**
     *  Lisää reseptilistaan uusia ohjeita
     */
    public void Lisaa2() {
        Ohje malli2 = new Ohje();
        malli2.Malliohje2();
        reseptit.lisaa(malli2);
        reseptit.TestiAinesosat2();
        paivita();
        ReseptiLista.setSelectedIndex(0);
    }
    
    
    /**
     * Tulostaa ohjeen tiedot
     * @param os tietovirta johon tulostetaan
     * @param ohje tulostettava ohje
     */
    public void tulosta(PrintStream os, final Ohje ohje) {
        os.println("----------------------------------------------");
        ohje.tulosta(os);
        os.println("----------------------------------------------");
        List<Ainesosa> ainesosat = reseptit.annaAinesosat(ohje);
        for (Ainesosa ain:ainesosat)
            ain.tulosta(os);
    }
    
    
    /**
     * Täyttää kenttiin tiedot ohjeista
     * @param id ohjeita
     */
    protected void tayta(int id) {
        int indeksi = id;  
        if ( indeksi <= 0 ) { 
            Ohje kohdalla = ohjeKohdalla; 
            if ( kohdalla != null ) indeksi = kohdalla.getId(); 
        }
        
        int k = jarjestys();
        String ehto = ohjehaku.getText(); 
        if (ehto.indexOf('*') < 0) ehto = "*" + ehto + "*"; 
        
        ReseptiLista.clear();

        int index = 0;
        Collection<Ohje> ohjeet;
        
            ohjeet = reseptit.etsi(ehto, k);
            for (Ohje ohje : ohjeet) {
                ReseptiLista.add(ohje.getNimi(), ohje);
            }
        
        ReseptiLista.setSelectedIndex(index);
    }
    
    
    /**
     * Tuottaa arvon jota käytetään lajittelussa
     * @return lajittelun määräävä arvo
     */
    public int jarjestys() {
        String jarjestys = Lajittelu.getSelectedText();
        switch ( jarjestys ) {
            case "Vanhin ensin": return 0;
            case "Ohjeen nimi": return 1;
            case "Annoskoko": return 2;
            default: return 0;
        }
    }
    
    
    /**
     * Näyttää valitun ohjeen tiedot
     */
    public void naytaTiedot() {
        ohjeKohdalla = ReseptiLista.getSelectedObject();
        if (ohjeKohdalla == null) return;
        
        chooserOhjeet.clear();
        OhjeArea.clear();
        TietoKentta.clear();

        
        List<Ainesosa> ainesosat = reseptit.annaAinesosat(ohjeKohdalla);
        for(Ainesosa ain : ainesosat) {
            String teksti = reseptit.getAnimi(ain) + " " + reseptit.getAmaara(ain) + " " + reseptit.getAyksikko(ain);
            chooserOhjeet.add(teksti, ain);
        }
        
        OhjeArea.setText(reseptit.getTeksti(ohjeKohdalla));
        TietoKentta.setText(reseptit.getNimi(ohjeKohdalla) + ": " + reseptit.getAnnoskoko(ohjeKohdalla) + " " + reseptit.getAnnostyyppi(ohjeKohdalla) + ", " + reseptit.getKesto(ohjeKohdalla));
        
    }
    
    
    /**
     * Poistetaan listasta valittuna oleva ohje
     */
    public void poistaOhje() {
        if ( ohjeKohdalla == null ) return;
        if ( !Dialogs.showQuestionDialog("Poisto", "Poistetaanko ohje: " + reseptit.getNimi(ohjeKohdalla), "Kyllä", "Ei") )
            return;
        reseptit.poista(ohjeKohdalla);
        int index = ReseptiLista.getSelectedIndex();
        tayta(0);
        ReseptiLista.setSelectedIndex(index);

    }
    
    
    /**
     * Kysyy ehtoja avaamiselle, jos niitä olisi
     * @return Voi
     */
    public boolean avaa() {
        return true;
    }

    
    /**
     * Tutkii tallennusta ja sen pohjalta päättää voiko sivun sulkea
     * @return Voi
     */
    public boolean voikoSulkea() {
        tallenna();
        return true;
    }


    @Override
    public String getResult() {
        // TODO Auto-generated method stub
        return null;
    }


    @Override
    public void handleShown() {
        // TODO Auto-generated method stub
        
    }


    @Override
    public void setDefault(String oletus) {
        // TODO Auto-generated method stub
        
    }


    
}
