/**
 * 
 */
package reseptit;


import java.io.OutputStream;
import java.io.PrintStream;
import fi.jyu.mit.ohj2.Mjonot;

/**
 * <pre>
 *|------------------------------------------------------------------------|
 *| Luokan nimi:  Ainesosa                             | Avustajat:        |
 *|-------------------------------------------------------------------------
 *| Vastuualueet:                                      |                   |
 *|                                                    |                   |
 *| - Tietää ainesosan kentät (nimi, määrä jne)        |                   |
 *| - osaa muuttaa 1| jauheliha | ..| merkkijonoksi    |                   |
 *|-------------------------------------------------------------------------
 * </pre>
 * @author Omistaja
 * @version 17.3.2025
 *
 */
public class Ainesosa {

        private String nimi = "";
        private int id = 0;
        private int ohjeNro;
        private double maara;
        private String yksikko;
        
        private static int seuraavaId = 1;
    
        
        /**
         * Parametriton muodostaja
         */
        public Ainesosa() {
            valitseId();
        }
        
        
        
        /**
         * Muodostaja jolle syötetäää ainesosan arvot
         * @param nimi ainesosan nimi
         * @param maara ainesosan määrä
         * @param yksikko ainesosna määrän yksikkö
         */
        public Ainesosa(String nimi, double maara, String yksikko) {
            this.nimi = nimi;
            this.maara = maara;
            this.yksikko = yksikko;
            valitseId();
        }
        
        
        /**
         * Muodostaja jolle annetaan ohjeen id johon tämä liittyy
         * @param ohjeNro ohje johon ainesosa liittyy
         */
        public Ainesosa(int ohjeNro) {
            this.ohjeNro = ohjeNro;
            valitseId();
        }
        
        
        /**
         * Muodostaja, jolle annetaan nimi
         * @param nimi ainesosalle annettava nimi
         */
        public Ainesosa(String nimi) {
            this.nimi = nimi;
            valitseId();
        }
        
        
        /**
         * Yhdistetään ainesosa ohjeeseen  
         * @param ohjeNro ohjeen viitenumero 
         */
        public void setOhjeNro(int ohjeNro) {
            this.ohjeNro = ohjeNro;
        }
        
        
        /**
         * Asettaa ainesosan nimen
         * @param nimi asetettava nimi
         */
        public void setNimi(String nimi) {
            this.nimi = nimi;
        }
        
        
        /**
         * Asettaa ainesoan määrän
         * @param maara asetettava määrä
         */
        public void setMaara(double maara) {
            this.maara = maara;
        }
        
        
        /**
         * Asettaa ainesosan määrän yksikön
         * @param yksikko asetettava yksikko
         */
        public void setYksikko(String yksikko) {
            this.yksikko = yksikko;
        }
        
        
        /**
         * Apumetodi, jolla saadaan täytetty testi arvot Ainesosalle.
         * @param nro viite ohjeeseen, johon ainesosaa tarvitaan
         */
        public void vastaaMaito(int nro) {
            ohjeNro = nro;
            nimi = "Maito";
            yksikko = "dl";
            maara = 2.5;
        }
        
        
        /**
         * Apumetodi, jolla saadaan täytettyä testiarvot Ainesosalle.
         * @param nro viite ohjeeseen, johon ainesosaa tarvitaan
         */
        public void vastaaSuola(int nro) {
            ohjeNro = nro;
            nimi = "Suola";
            yksikko = "tl";
            maara = 0.5;
        }
        
        
        /**
         * Apumetodi, jolla saadaan täytettyä testiarvot Ainesosalle.
         * @param nro viite ohjeeseen, johon ainesosaa tarvitaan
         */
        public void vastaaVehnajauho(int nro) {
            ohjeNro = nro;
            nimi = "Vehnajauho";
            yksikko = "dl";
            maara = 8;
        }


        
        /**
         * @return palauttaa ainesosan nimen
         */
        public String getNimi() {
            return this.nimi;
        }
        
        
        /**
         * @return palauttaa määrän
         */
        public double getMaara() {
            return this.maara;
        }
        
        
        /**
         * @return palauttaa ainesosan yksikön
         */
        public String getYksikko() {
            return this.yksikko;
        }
                
        /**
         * Tulostaa ainesosan tiedot
         * @param os tietovirta mihin tulostetaan
         */
        public void tulosta(OutputStream os) {
            tulosta(new PrintStream(os));
        }
        
        
        /**
         * Tulostaa ainesosan tiedot
         * @param out tietovirta mihin tulostetaan
         */
        public void tulosta(PrintStream out) {
            out.println(String.format("%03d", id) + " " + nimi + " " + maara + " " + yksikko);
        }
        
        
        /**
         * Antaa ainesosalle tunnusnumeron
         * @return palauttaa ainesosan id:n
         * @example
         * <pre name="test">
         * Ainesosat Ainesosat = new Ainesosat();
         * Ainesosa maito3 = new Ainesosa();
         * Ainesosa maito4 = new Ainesosa();
         * maito3.getId() === 1;
         * Ainesosat.lisaa(maito3);
         * Ainesosat.lisaa(maito4);
         * int n1 = maito3.getId();
         * int n2 = maito4.getId();
         * n1 === n2-1;
         * </pre>
         */
        public int valitseId() {
            this.id = seuraavaId;
            seuraavaId++;
            return this.id;
        }
        
        
        /**
         * Poimii aineosan numeron
         * @return palauttaa ainesosan numeron
         */
        public int getId() {
            return id;
        }
        
        
        /**
         * Palautetaan missä ohjeessa ainesosaa käytetään
         * @return ohjeen id
         */
        public int getOhjeNro() {
            return ohjeNro;
        }

    
    /** Asettaa id numeron ja samalla varmistaa että
     * seuraava numero on aina suurempi kuin tähän mennessä suurin.
     * @param nr asetettava tunnusnumero
     */
    private void setId(int nr) {
        id = nr;
        if ( id >= seuraavaId ) seuraavaId = id + 1;
    }


    /**
     * Palauttaa ainesosan tiedot merkkijonona jonka voi tallentaa tiedostoon.
     * @return ainesosa tolppaeroteltuna merkkijonona 
     * @example
     * <pre name="test">
     *   Ainesosa ainesosa = new Ainesosa();
     *   ainesosa.parse("   2   |  Maito  |   1  | 8 | dl ");
     *   ainesosa.toString()    === "2|Maito|1|8.0|dl";
     * </pre>
     */
    @Override
    public String toString() {
        return "" + id + "|" + nimi + "|" + ohjeNro + "|" + maara + "|" + yksikko;
    }


    /**
     * Selvitää ainesosan tiedot | erotellusta merkkijonosta.
     * Pitää huolen että seuraavaNro on suurempi kuin tuleva tunnusnro.
     * @param rivi josta ainesosan tiedot otetaan
     * @example
     * <pre name="test">
     * Ainesosat Ainesosat = new Ainesosat();
     *   Ainesosa ainesosa = new Ainesosa();
     *   ainesosa.parse("   2   |  Maito  |   1  | 8 | dl ");
     *   ainesosa.getOhjeNro() === 1;
     *   ainesosa.toString()    === "2|Maito|1|8.0|dl";
     *   
     *   Ainesosat.lisaa(ainesosa);
     *   int n = ainesosa.getId();
     *   ainesosa.parse(""+(n+20));
     *   Ainesosat.lisaa(ainesosa);
     *   ainesosa.getId() === n+20;
     *   ainesosa.toString()     === "" + (n+20) + "|Maito|1|8.0|dl";
     * </pre>
     */
    public void parse(String rivi) {
        StringBuffer sb = new StringBuffer(rivi);
        setId(Mjonot.erota(sb, '|', getId()));
        nimi = Mjonot.erota(sb, '|', nimi);
        ohjeNro = Mjonot.erota(sb, '|', ohjeNro);
        maara = Mjonot.erota(sb, '|', maara);
        yksikko = Mjonot.erota(sb, '|', yksikko);
    }


    @Override
    public boolean equals(Object obj) {
        if ( obj == null ) return false;
        return this.toString().equals(obj.toString());
    }
    

    @Override
    public int hashCode() {
        return id;
    }


    /**
    * Testiohjelma Harrastukselle.
     * @param args ei käytössä
     */
    public static void main(String[] args) {
        Ainesosa ain = new Ainesosa();
        ain.parse("   2   |  Maito  |   1  | 8 | dl ");
        ain.vastaaMaito(2);
        ain.tulosta(System.out);
    
    }

}
