/**
 * 
 */
package reseptit;

import java.io.OutputStream;
import java.io.PrintStream;
import java.util.Comparator;

import fi.jyu.mit.ohj2.Mjonot;

/**
 * <pre>
* |------------------------------------------------------------------------|
* | Luokan nimi:    Ohje                              | Avustajat:         |
* |-------------------------------------------------------------------------
* | Vastuualueet:                                      |                   |
* |                                                    |                   |
* |(- ei tiedä reseptistä eikä käyttöliittymästä       |                   |
* | - tietää ohjeen kentät (nimi, kesto, määrä jne.)   |                   |
* | - osaa muuttaa 1| makaroonilaatikko|..|            |                   |
* | merkkijonoksi|                                     |                   |
* |-------------------------------------------------------------------------
 * </pre>
 * @author Omistaja
 * @version 13.3.2025
 *
 */
public class Ohje implements Cloneable{

    private String nimi = "";
    private int id = 0;
    private String kesto = "";
    private int annoskoko = 1;
    private String annostyyppi = "annosta";
    private String teksti = "";
    
    private static int seuraavaId = 1;
    
    
    /**
     * Parametriton muodostaja
     */
    public Ohje() {
        valitseId();
    }
    
    
    /**
     * Ohjeen muodostaja jolle annetaan kaikki annettavat tiedot
     * @param nimi ohjeen nimi
     * @param annoskoko ohjeen tuottama annosmäärä
     * @param annostyyppi annoksen suuruutta kuvaava tyyppi
     * @param kesto kauanko ruuan teossa kestää
     * @param teksti ohjeen tekstiosa
     */
    public Ohje(String nimi, int annoskoko, String annostyyppi, String kesto, String teksti) {
        this.nimi = nimi;
        this.annoskoko = annoskoko;
        this.annostyyppi = annostyyppi;
        this.kesto = kesto;
        this.teksti = teksti;
        valitseId();
    }
    
    
    /**
     * Harjoittelua ja mallinnusta varten valmiiksi täytetty
     */
    public void Malliohje() {
        this.nimi = "Pinaattikeitto";
        this.kesto = "lyhyt";
        this.annoskoko = 2;
        this.annostyyppi = "annos";
        this.teksti = "Sulata voi kattilassa. Lisää jauhot hyvin sekoittaen. Anna kiehua hetken.\r\n"
                + "Lisää maito ohuena nauhana koko ajan sekoittaen. Kuumenna samalla kiehuvaksi.\r\n"
                + "Lisää pinaatti. Anna kiehua hiljalleen muutama minuutti välillä sekoittaen.\r\n"
                + "Mausta. Älä keitä suolan lisäämisen jälkeen.\r\n"
                + "Lisää lautaselle keiton pinnalle halkaistut, keitetyt munat.\r\n"
                + "Tarjoa pinaattikeitto ruisleivän kanssa.";
    }
    
    
    /**
     * Harjoittelua ja mallinnusta varten valmiiksi täytetty
     */
    public void Malliohje2() {
        this.nimi = "Makaroonilaatikko";
        this.kesto = "normaali";
        this.annoskoko = 4;
        this.annostyyppi = "annos";
        this.teksti = "Keitä makaronit pakkauksen ohjeen mukaan. Voit jättää makaronit myös keittämättä ja pääset näin helpommalla. Ruskista jauheliha ja sipuli pannulla paistamalla. Mausta kypsä jauheliha.\r\n"
                + "Kaada kerroksittain makaroneja ja jauhelihaa voideltuun vuokaan. Sekoita kananmunat ja maito (tai lihaliemi) ja kaada vuokaan. \r\n"
                + "Kypsennä 200-asteisen uunin alatasolla 1 tunti. Mikäli et esikeittänyt makaroneja, sekoita seosta kerran tai pari paiston alkupuolella/puolivälissä, jotta makaronit eivät jää painuneina vuoan pohjalle. ";
    }
    
    
    @Override
    public Ohje clone() throws CloneNotSupportedException {
        Ohje uusi;
        uusi = (Ohje) super.clone();
        return uusi;    
    }
    
    
    /**
     * Tulostaa ohjeen tiedot
     * @param os tietovirta mihin tulostetaan
     */
    public void tulosta(OutputStream os) {
        tulosta(new PrintStream(os));
    }
    
    
    /**
     * Tulostaa ohjeen tiedot
     * @param out tietovirta mihin tulostetaan
     */
    public void tulosta(PrintStream out) {
        out.println(String.format("%03d", id) + " " + nimi + ", " + kesto + ", " + annoskoko + ", " + annostyyppi + "\n" + teksti);
    }
    
    
    /**
     * Antaa ohjeelle tunnusnumeron
     * @return palauttaa ohjeen id:n
     * @example
     * <pre name="test">
     * Ohje makaronilaatikko3 = new Ohje();
     * Ohje makaronilaatikko4 = new Ohje();
     * makaronilaatikko3.getId() === 1;
     * int n1 = makaronilaatikko3.getId();
     * int n2 = makaronilaatikko4.getId();
     * n1 === n2-1;
     * </pre>
     */
    public int valitseId() {
        this.id = seuraavaId;
        seuraavaId++;
        return this.id;
    }
    
    
    /**
     * Poimii ohjeen numeron
     * @return palauttaa ohjeen numeron
     */
    public int getId() {
        return id;
    }
    
    
    /**
     * Asettaa id:n ja samalla varmistaa että
     * seuraava id on aina suurempi kuin tähän mennessä suurin.
     * @param nr asetettava id
     */
    public void setId(int nr) {
        id = nr;
        if (id >= seuraavaId) seuraavaId = id + 1;
    }
    
    
    /**
     * Asettaa ohjeen nimen
     * @param nimi asetettava nimi
     */
    public void setNimi(String nimi) {
        this.nimi = nimi;
    }
    
    
    /**
     * Asettaa ohjeen annoskoon
     * @param annoskoko ohjeen annoskoko
     */
    public void setAnnosKoko(int annoskoko) {
        this.annoskoko = annoskoko;
    }
    
    
    /**
     * Asettaa annokselle tyypin
     * @param annostyyppi asetettava annostyyppi
     */
    public void setAnnosTyyppi(String annostyyppi) {
        this.annostyyppi = annostyyppi;
    }
    
    
    /**
     * Asettaa arvion ohjeen kestosta
     * @param kesto arivo kestosta
     */
    public void setKesto(String kesto) {
        this.kesto = kesto;
    }
    
    
    /**
     * Asettaa ohjeelle teksin eli ohjeen tekstiosuuden
     * @param teksti asetettava ohjeen teksti
     */
    public void setTeksti(String teksti) {
        this.teksti = teksti;
    }
        
    
    @Override
    public boolean equals(Object ohje) {
        if ( ohje == null ) return false;
        return this.toString().equals(ohje.toString());
    }


    @Override
    public int hashCode() {
        return id;
    }

    
    /**
     * Palauttaa ohjeen tiedot merkkijonona jonka voi tallentaa tiedostoon.
     * @return ohje tolppaeroteltuna merkkijonona 
     * @example
     * <pre name="test">
     *   Ohje ohje = new Ohje();
     *   ohje.parse("   1  |  Makaronilaatikko   | keskipitkä");
     *   ohje.toString().startsWith("1|Makaronilaatikko|keskipitkä") === true;
     * </pre>  
     */
    @Override
    public String toString() {
        return "" +
                getId() + "|" +
                nimi + "|" +
                kesto + "|" +
                annoskoko + "|" +
                annostyyppi + "|" +
                teksti;
    }
    
    
    /**
     * Selvitää ohjeen tiedot | erotellusta merkkijonosta
     * Pitää huolen että seuraavaId on suurempi kuin tuleva Id.
     * @param rivi josta ohjeen tiedot otetaan
     * 
     * @example
     * <pre name="test">
     *   Ohjeet ohjeet = new Ohjeet();
     *   Ohje ohje = new Ohje();
     *   ohje.parse("   1  |  Makaronilaatikko   | keskipitkä");
     *   ohje.getId() === 1;
     *   ohje.toString().startsWith("1|Makaronilaatikko|keskipitkä|") === true; 
     *   ohjeet.lisaa(ohje);
     *   int n = ohje.getId();
     *   ohje.parse(""+(n+20));       // Otetaan merkkijonosta vain tunnusnumero
     *   ohjeet.lisaa(ohje);           // ja tarkistetaan että seuraavalla kertaa tulee yhtä isompi
     *   ohje.getId() === n+20;
     *     
     * </pre>
     */
    public void parse(String rivi) {
        StringBuffer sb = new StringBuffer(rivi);
        setId(Mjonot.erota(sb, '|', getId()));
        nimi = Mjonot.erota(sb, '|', nimi);
        kesto = Mjonot.erota(sb, '|', kesto);
        annoskoko = Mjonot.erota(sb, '|', annoskoko);
        annostyyppi = Mjonot.erota(sb, '|', annostyyppi);
        teksti = Mjonot.erota(sb, '|', teksti);
    }

    
    /**
     * poimii ohjeen nimen
     * @return palauttaa ohjeen nimen
     */
    public String getNimi() {
        return nimi;
    }
    
    
    /**
     * @return annoskoko
     */
    public int getAnnoskoko() {
        return this.annoskoko;
    }
    
    
    /**
     * @return annostyyppi
     */
    public String getAnnostyyppi() {
        return this.annostyyppi;
    }
    
    
    /**
     * @return kesto
     */
    public String getKesto() {
        return this.kesto;
    }
    
    
    /**
     * @return teksti
     */
    public String getTeksti() {
        return this.teksti;
    }
    
    
    /**
    * @param k monenenko kentän sisältö palautetaan
    * @return kentän sisältö merkkijonona
    */
   public String anna(int k) {
       switch (k) {
       case 0: return "" + id;
       case 1: return nimi;
       case 2: return "" + annoskoko;
       case 3: return annostyyppi;
       case 4: return kesto;
       case 5: return teksti;
       default: return "";
       }
   }

    
   /**
 * @author Omistaja
 * @version 24.4.2025
 * Vertailun suorittava luokka
 */
public static class Vertailija implements Comparator<Ohje> { 
        private int k;  
            
        @SuppressWarnings("javadoc") 
        public Vertailija(int k) { 
          this.k = k; 
        } 
                
        /**
         * Vertaa kahta ohjetta toisiinsa
         * @param ohje1 ensimmäinen verrattava ohje
         * @param ohje2 toinen verrattava ohje
         * @return luvun 1, 0 tai -1 riippuen verrattavien suhteesta
         */
        @Override
        public int compare(Ohje ohje1, Ohje ohje2) { 
           if (k != 1) {
               if (Integer.parseInt(ohje1.getAvain(k)) > Integer.parseInt(ohje2.getAvain(k))) return 1;
               if (Integer.parseInt(ohje1.getAvain(k)) == Integer.parseInt(ohje2.getAvain(k))) return 0;
               if (Integer.parseInt(ohje1.getAvain(k)) < Integer.parseInt(ohje2.getAvain(k))) return -1;
           }
           return ohje1.getAvain(k).compareToIgnoreCase(ohje2.getAvain(k)); 
        } 
    } 
   
   
   /** 
   * Antaa k:n kentän sisällön merkkijonona 
   * @param k monenenko kentän sisältö palautetaan 
   * @return kentän sisältö merkkijonona 
   */ 
   public String getAvain(int k) { 
      switch ( k ) { 
        case 0: return "" + id;
        case 1: return "" + nimi.toUpperCase();
        case 2: return "" + annoskoko;  
        default: return "Väärin syötetty"; 
       } 
   }
   
   
    /**
     * @param args ei käytössä
     */
    public static void main(String[] args) {
        Ohje makaronilaatikko = new Ohje("Makaroonilaatikkko", 4, "pitkä", "annos", "Yhdistele aineet");
        Ohje makaronilaatikko2 = new Ohje("Makaroonilaatikko2", 4, "pitkä", "annos", "Yhdistele aineet");
        Ohje malli = new Ohje();
        malli.Malliohje();
        
        makaronilaatikko.tulosta(System.out);
        makaronilaatikko2.tulosta(System.out);
        malli.tulosta(System.out);
        
        makaronilaatikko.tulosta(System.out);
        makaronilaatikko2.tulosta(System.out);
        
    }

}
