/**
 * 
 */
package reseptit;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;
import java.util.NoSuchElementException;

import fi.jyu.mit.ohj2.WildChars;


/**
 * <pre>
 *|------------------------------------------------------------------------|
 *| Luokan nimi:  Ohjeet                               | Avustajat:        |
 *|-------------------------------------------------------------------------
 *| Vastuualueet:                                      |                   |
 *|                                                    | - Ohje            |
 *| - Pitää listaa ohjeista, sekä osaa lisätä ja       |                   |
 *| poistaa ohjeita                                    |                   |
 *| - Lukee ja kirjoitaa ohjeista tiedostoon           |                   |
 *| - Osaa etsiä ja lajitella ohjeita                  |                   |
 *|-------------------------------------------------------------------------
 * </pre>
 * @author Omistaja
 * @version 18.3.2025
 *
 */
public class Ohjeet implements Iterable<Ohje>{
    
    private int MaxOhjeet = 5;
    private boolean muutettu = false;
    private Ohje alkiot[] = new Ohje[MaxOhjeet];
    private int lkm = 0;
    private String tiedostonPerusNimi = "ohjeet";
    private String kokoNimi = "";
    
    
    /**
     * Joku konstruktori
     */
    public Ohjeet() {
        
    }
    
    
    /**
     * Lisää uuden ohjeen tietorakenteeseen.  Ottaa ohjeen omistukseensa.
     * @param ohje lisätäävän ohjeeseen viite.  Huom tietorakenne muuttuu omistajaksi
     * @example
     * <pre name="test">
     * #THROWS SailoException 
     * Ohjeet ohjeet = new Ohjeet();
     * Ohje makaronilaatikko1 = new Ohje(), makaronilaatikko2 = new Ohje();
     * ohjeet.getLkm() === 0;
     * ohjeet.lisaa(makaronilaatikko1); ohjeet.getLkm() === 1;
     * ohjeet.lisaa(makaronilaatikko2); ohjeet.getLkm() === 2;
     * ohjeet.lisaa(makaronilaatikko1); ohjeet.getLkm() === 3;
     * ohjeet.anna(0) === makaronilaatikko1;
     * ohjeet.anna(1) === makaronilaatikko2;
     * ohjeet.anna(2) === makaronilaatikko1;
     * ohjeet.anna(1) == makaronilaatikko1 === false;
     * ohjeet.anna(1) == makaronilaatikko2 === true;
     * ohjeet.anna(3) === makaronilaatikko1; #THROWS IndexOutOfBoundsException 
     * ohjeet.lisaa(makaronilaatikko1); ohjeet.getLkm() === 4;
     * ohjeet.lisaa(makaronilaatikko1); ohjeet.getLkm() === 5;
     * ohjeet.lisaa(makaronilaatikko1);
     * </pre>
     */
    public void lisaa(Ohje ohje) {
        if (lkm <= alkiot.length-1) {
            alkiot[lkm] = ohje;
            lkm++;
            muutettu = true;
        }
        else {
            this.MaxOhjeet = MaxOhjeet + 5;
            Ohje uudetAlkiot[] = new Ohje[MaxOhjeet];
            for (int i=0; i<MaxOhjeet-5; i++) {
                uudetAlkiot[i]=alkiot[i];
            }
            this.alkiot = uudetAlkiot;
            alkiot[lkm] = ohje;
            lkm++;
            muutettu = true;
        }
    }
    
    
    /**
     * Palauttaa viitteen i:teen ohjeeseen.
     * @param i monennenko ohjeen viite halutaan
     * @return viite ohjeeseen, jonka indeksi on i
     * @throws IndexOutOfBoundsException jos i ei ole sallitulla alueella  
     */
    public Ohje anna(int i) throws IndexOutOfBoundsException {
        if (i < 0 || lkm <= i) throw new IndexOutOfBoundsException("Laiton indeksi: " + i);
        return alkiot[i];
    }


    /**
     * Lukee Ohjeet tiedostosta.
     * @param tied tiedoston hakemisto
     * @throws SailoException jos lukeminen epäonnistuu
     */
    public void lueTiedostosta(String tied) throws SailoException {
        setTiedPerusNimi(tied);
        try ( BufferedReader fi = new BufferedReader(new FileReader(getTiedostonNimi())) ) {
            kokoNimi = fi.readLine();
            if ( kokoNimi == null ) throw new SailoException("Reseptikirjan nimi puuttuu");
            String rivi = fi.readLine();
            if ( rivi == null ) throw new SailoException("Maksimikoko puuttuu");
            // int maxKoko = Mjonot.erotaInt(rivi,10); // tehdään jotakin

            while ( (rivi = fi.readLine()) != null ) {
                rivi = rivi.trim();
                if ( "".equals(rivi) || rivi.charAt(0) == ';' ) continue;
                Ohje ohje = new Ohje();
                ohje.parse(rivi); // voisi olla virhekäsittely
                lisaa(ohje);
            }
            muutettu = false;
        } catch ( FileNotFoundException e ) {
            throw new SailoException("Tiedosto " + getTiedostonNimi() + " ei aukea");
        } catch ( IOException e ) {
            throw new SailoException("Ongelmia tiedoston kanssa: " + e.getMessage());
        }
    }
    
    /**
    * Luetaan aikaisemmin annetun nimisestä tiedostosta
    * @throws SailoException jos tulee poikkeus
    */
   public void lueTiedostosta() throws SailoException {
       lueTiedostosta(getTiedostonPerusNimi());
   }


   /**
    * Tallentaa ohjeet tiedostoon.  
    * Tiedoston muoto:
    * <pre>
    * Reseptikirja
    * 20
    * ; kommenttirivi
    * 2|Pinaattikeitta|nopea|2 annosta|Sulata voi kattilassa. Lisää jauhot hyvin sekoittaen. Anna kiehua hetken.
Lisää maito ohuena nauhana koko ajan sekoittaen. Kuumenna samalla kiehuvaksi.
Lisää pinaatti. Anna kiehua hiljalleen muutama minuutti välillä sekoittaen.
Mausta. Älä keitä suolan lisäämisen jälkeen.
Lisää lautaselle keiton pinnalle halkaistut, keitetyt munat.
Tarjoa pinaattikeitto ruisleivän kanssa.
    * 3|Kinkkukiusaus|keskipitkä|2 annosta|Sekoita uunivuoassa peruna-sipulisekoitus ja mausteet. Sekoita joukkoon kinkku. Kaada päälle ruokakerma
Kypsennä uunin ala- / keskiosassa 225 asteessa n. 1 tunti tai kunnes perunat ovat kypsiä (tarkista aika perunapakkauksesta). Peitä vuoka leivinpaperilla, jos pinta tummuu liikaa.
Ripota kypsän kiusauksen päälle hienonnettua persiljaa. Tarjoa raikkaan salaatin kanssa.
    * </pre>
    * @throws SailoException jos talletus epäonnistuu
    */

    public void tallenna() throws SailoException {
        if ( !muutettu ) return;

        File fbak = new File(getBakNimi());
        File ftied = new File(getTiedostonNimi());
        fbak.delete(); // if .. System.err.println("Ei voi tuhota");
        ftied.renameTo(fbak); // if .. System.err.println("Ei voi nimetä");

        try ( PrintWriter fo = new PrintWriter(new FileWriter(ftied.getCanonicalPath())) ) {
            fo.println(getKokoNimi());
            fo.println(alkiot.length);
            for (Ohje ohje : this) {
                fo.println(ohje.toString());
            }
            //} catch ( IOException e ) { // ei heitä poikkeusta
            //  throw new SailoException("Tallettamisessa ongelmia: " + e.getMessage());
        } catch ( FileNotFoundException ex ) {
            throw new SailoException("Tiedosto " + ftied.getName() + " ei aukea");
        } catch ( IOException ex ) {
            throw new SailoException("Tiedoston " + ftied.getName() + " kirjoittamisessa ongelmia");
        }

        muutettu = false;
    }

    
    /**
     * Palauttaa Reseptikirjan koko nimen
     * @return Reseptikirjan koko nimi merkkijononna
     */
    public String getKokoNimi() {
        return kokoNimi;
    }


    /**
     * Palauttaa Reseptien lukumäärän
     * @return ohjeiden lukumäärä
     */
    public int getLkm() {
        return lkm;
    }
    
    
    /**
     * Palauttaa tiedoston nimen, jota käytetään tallennukseen
     * @return tallennustiedoston nimi
     */
    public String getTiedostonPerusNimi() {
        return tiedostonPerusNimi;
    }


    /**
     * Asettaa tiedoston perusnimen ilman tarkenninta
     * @param nimi tallennustiedoston perusnimi
     */
    public void setTiedPerusNimi(String nimi) {
        tiedostonPerusNimi = nimi;
    }


    /**
     * Palauttaa tiedoston nimen, jota käytetään tallennukseen
     * @return tallennustiedoston nimi
     */
    public String getTiedostonNimi() {
        return getTiedostonPerusNimi() + ".dat";
    }


    /**
     * Palauttaa varakopiotiedoston nimen
     * @return varakopiotiedoston nimi
     */
    public String getBakNimi() {
        return tiedostonPerusNimi + ".bak";
    }


    /**
     * Luokka ohjeiden iteroimiseksi.
     * @example
     * <pre name="test">
     * #THROWS SailoException 
     * #PACKAGEIMPORT
     * #import java.util.*;
     * 
     * Ohjeet ohjeet = new Ohjeet();
     * Ohje kinkkukiusaus1 = new Ohje(), kinkkukiusaus2 = new Ohje();
     * ohjeet.lisaa(kinkkukiusaus1); ohjeet.lisaa(kinkkukiusaus2);
     *
     * ohjeet.lisaa(kinkkukiusaus1); 
     * ohjeet.lisaa(kinkkukiusaus2); 
     * ohjeet.lisaa(kinkkukiusaus1); 
     * 
     * StringBuffer ids = new StringBuffer(30);
     * for (Ohje ohje:ohjeet)   // Kokeillaan for-silmukan toimintaa
     *   ids.append(" "+ohje.getId());           
     * 
     * String tulos = " " + kinkkukiusaus1.getId() + " " + kinkkukiusaus2.getId() + " " + kinkkukiusaus1.getId() + " " +  kinkkukiusaus2.getId() + " " + kinkkukiusaus1.getId() ;
     * 
     * ids.toString() === tulos; 
     * 
     * ids = new StringBuffer(30);
     * for (Iterator<Ohje>  i=ohjeet.iterator(); i.hasNext(); ) { // ja iteraattorin toimintaa
     *   Ohje ohje = i.next();
     *   ids.append(" "+ohje.getId());           
     * }
     * 
     * ids.toString() === tulos;
     * 
     * Iterator<Ohje>  i=ohjeet.iterator();
     * i.next() == kinkkukiusaus1  === true;
     * i.next() == kinkkukiusaus2  === true;
     * i.next() == kinkkukiusaus1  === true;
     * 
     * i.next(); 
     *  
     * </pre>
     */
    public class OhjeetIterator implements Iterator<Ohje> {
        private int kohdalla = 0;


        /**
         * Onko olemassa vielä seuraavaa ohjetta
         * @see java.util.Iterator#hasNext()
         * @return true jos on vielä ohjeita
         */
        @Override
        public boolean hasNext() {
            return kohdalla < getLkm();
        }


        /**
         * Annetaan seuraava ohje
         * @return seuraava ohje
         * @throws NoSuchElementException jos seuraava alkiota ei enää ole
         * @see java.util.Iterator#next()
         */
        @Override
        public Ohje next() throws NoSuchElementException {
            if ( !hasNext() ) throw new NoSuchElementException("Ei oo");
            return anna(kohdalla++);
        }


        /**
         * Tuhoamista ei ole toteutettu
         * @throws UnsupportedOperationException aina
         * @see java.util.Iterator#remove()
         */
        @Override
        public void remove() throws UnsupportedOperationException {
            throw new UnsupportedOperationException("Me ei poisteta");
        }
    }


    /**
     * Palautetaan iteraattori ohjeista.
     * @return ohje iteraattori
     */
    @Override
    public Iterator<Ohje> iterator() {
        return new OhjeetIterator();
    }


    /** 
     * Palauttaa "taulukossa" hakuehtoon vastaavien ohjeitten viitteet 
     * @param hakuehto hakuehto 
     * @param k etsittävän kentän indeksi  
     * @return tietorakenteen löytyneistä ohjeista
     * @example 
     * <pre name="test">  
     *   Ohjeet ohjeet = new Ohjeet(); 
     *   Ohje ohje1 = new Ohje(); ohje1.parse("1|Makaronilaatikko|keskipitkä|Keitä makaronit pakkauksen ohjeen mukaan. Voit jättää makaronit myös keittämättä ja pääset näin helpommalla. Ruskista jauheliha ja sipuli pannulla paistamalla. Mausta kypsä jauheliha. Kaada kerroksittain makaroneja ja jauhelihaa voideltuun vuokaan. Sekoita kananmunat ja maito (tai lihaliemi) ja kaada vuokaan. Kypsennä 200-asteisen uunin alatasolla 1 tunti. Mikäli et esikeittänyt makaroneja, sekoita seosta kerran tai pari paiston alkupuolella/puolivälissä, jotta makaronit eivät jää painuneina vuoan pohjalle."); 
     *   Ohje ohje2 = new Ohje(); ohje2.parse("2|Pinaattikeitto|nopea|Sulata voi kattilassa. Lisää jauhot hyvin sekoittaen. Anna kiehua hetken. Lisää maito ohuena nauhana koko ajan sekoittaen. Kuumenna samalla kiehuvaksi. Lisää pinaatti. Anna kiehua hiljalleen muutama minuutti välillä sekoittaen. Mausta. Älä keitä suolan lisäämisen jälkeen. Lisää lautaselle keiton pinnalle halkaistut, keitetyt munat. Tarjoa pinaattikeitto ruisleivän kanssa."); 
     *   Ohje ohje3 = new Ohje(); ohje3.parse("3|Kinkkukiusaus|keskpitkä|Sekoita uunivuoassa peruna-sipulisekoitus ja mausteet. Sekoita joukkoon kinkku. Kaada päälle ruokakerma Kypsennä uunin ala- / keskiosassa 225 asteessa n. 1 tunti tai kunnes perunat ovat kypsiä (tarkista aika perunapakkauksesta). Peitä vuoka leivinpaperilla, jos pinta tummuu liikaa. Ripota kypsän kiusauksen päälle hienonnettua persiljaa. Tarjoa raikkaan salaatin kanssa."); 
     *   Ohje ohje4 = new Ohje(); ohje4.parse("4|Korvapuustit|hidas|Liuota hiiva kädenlämpöiseen maitoon. Lisää munat, sokeri, mausteet ja osa jauhoista. Sekoita taikina tasaiseksi. Alusta vähitellen loput jauhot taikinaan, kunnes taikina irtoaa kulhon reunoista. Lisää lopuksi sulatettu Oivariini. Anna taikinan kohota liinan alla vedottomassa paikassa kaksinkertaiseksi. Kauli taikina kahdeksi 60 x 30 cm:n kokoiseksi levyksi. Levitä levyille pehmeä Oivariini. Ripota pinnalle sokeri ja kaneli. Kääri taikinalevyt rulliksi pitkältä sivulta. Jätä saumakohta alapuolelle. Leikkaa rullat terävällä veitsellä vinottain kolmikulmaisiksi paloiksi. Nosta palat pystyyn kapea kohta ylöspäin. Paina sormilla kapea kohta alas pöytään saakka. Näin leikkauspinnat nousevat ylös. Kohota liinan alla. Voitele kohonneet puustit. Ripota pinnalle mantelirouhetta tai raesokeria. Paista uunin keskiosassa 225 asteessa n. 8 min. ");  
     *   ohjeet.lisaa(ohje1); ohjeet.lisaa(ohje2); ohjeet.lisaa(ohje3); ohjeet.lisaa(ohje4);
     * </pre> 
     */ 
    public Collection<Ohje> etsi(String hakuehto, int k) {
        String ehto = "*";
        if (hakuehto != null && hakuehto.length() > 0) ehto = hakuehto;
        int hk = k;
        if (hk < 0) hk = 0;
        List<Ohje> loytyneet = new ArrayList<Ohje>(); 
        for (Ohje ohje : this) { 
            if (WildChars.onkoSamat(ohje.anna(1), ehto)) 
                loytyneet.add(ohje);  
        } 
        Collections.sort(loytyneet, new Ohje.Vertailija(hk));
        return loytyneet; 
    }
    
    
    /**
     * @param ohje ohje joka korvataan muokatulla ohjeella
     */
    public void korvaa(Ohje ohje) {
            int id = ohje.getId();
            for (int i = 0; i < lkm; i++) {
                if ( alkiot[i].getId() == id ) {
                    alkiot[i] = ohje;
                    muutettu = true;
                    return;
                }
            }
            lisaa(ohje);
    }
    
    
    /** 
     * Etsii ohjeen id:n perusteella 
     * @param id tunnusnumero, jonka mukaan etsitään 
     * @return löytyneen ohjeen indeksi tai -1 jos ei löydy 
     * <pre name="test"> 
     * #THROWS SailoException  
     * Ohjeet ohjeet = new Ohjeet(); 
     * Ohje makaronilaatikko1 = new Ohje(), makaronilaatikko2 = new Ohje(), makaronilaatikko3 = new Ohje(); 
     * makaronilaatikko1.lisaa(); makaronilaatikko2.lisaa(); makaronilaatikko3.lisaa(); 
     * int id1 = makaronilaatikko1.getId(); 
     * ohjeet.lisaa(makaronilaatikko1); ohjeet.lisaa(makaronilaatikko2); ohjeet.lisaa(makaronilaatikko3); 
     * ohjeet.etsiId(id1+1) === 1; 
     * Ohjeet.etsiId(id1+2) === 2; 
     * </pre> 
     */ 
    public int etsiId(int id) { 
        for (int i = 0; i < lkm; i++) 
            if (id == alkiot[i].getId()) return i; 
        return -1; 
    }


    /** 
     * Poistaa ohjeen jolla on valittu tunnusnumero  
     * @param id poistettavan ohjeen id 
     * @return 1 jos poistettiin, 0 jos ei löydy 
     * @example 
     * <pre name="test"> 
     * #THROWS SailoException  
     * Ohjeet ohjeet = new Ohjeet(); 
     * Ohje makaronilaatikko1 = new Ohje(), makaronilaatikko2 = new Ohje(), makaronilaatikko3 = new Ohje(); 
     * makaronilaatikko1.lisaa(); makaronilaatikko2.lisaa(); makaronilaatikko3.lisaa(); 
     * int id1 = makaronilaatikko1.getId(); 
     * ohjeet.lisaa(makaronilaatikko1); ohjeet.lisaa(makaronilaatikko2); ohjeet.lisaa(makaronilaatikko3); 
     * ohjeet.poista(id1+1) === 1; 
     * ohjeet.annaId(id1+1) === null; ohjeet.getLkm() === 2; 
     * ohjeet.poista(id1) === 1; ohjeet.getLkm() === 1; 
     * ohjeet.poista(id1+3) === 0; ohjeet.getLkm() === 1; 
     * </pre> 
     */ 
    public int poista(int id) { 
        int ind = etsiId(id); 
        if (ind < 0) return 0; 
        lkm--; 
        for (int i = ind; i < lkm; i++) 
            alkiot[i] = alkiot[i + 1]; 
        alkiot[lkm] = null; 
        muutettu = true; 
        return 1; 
    } 
    
    
    /**
     * @param args ei käytössä
     */
    public static void main(String[] args) {
        Ohjeet ohjeet = new Ohjeet();

        Ohje makaronilaatikko = new Ohje("makaronilaatikko", 9, "annos", "pitkä", "sekoita aineet"), makaronilaatikko2 = new Ohje("makaronilaatikko", 9, "annos", "pitkä", "sekoita aineet");
       


        System.out.println("============= Ohjeet testi =================");
        System.out.println(makaronilaatikko);
        System.out.println(makaronilaatikko2);
        for (int i = 0; i < ohjeet.getLkm(); i++) {
            Ohje ohje = ohjeet.anna(i);
            System.out.println("Ohjeetn nro: " + i);
            ohje.tulosta(System.out);
        }

    }

}
