/**
 * 
 */
package reseptit;

import java.util.List;
import java.io.File;
import java.util.Collection;


/**
 * <pre>
 *|------------------------------------------------------------------------|
 *| Luokan nimi:  Reseptit                             | Avustajat:        |
 *|-------------------------------------------------------------------------
 *| Vastuualueet:                                      |                   |
 *|                                                    | - Ohje            |
 *| - Huolehtii Ainesosat ja Ohjeet välisestä          | - Ohjeet          |
 *| yhteistyöstä ja välittää näiden tietoja            | - Ainesosa        |
 *| pyydettäessä                                       | - Ainesosat       |
 *|-------------------------------------------------------------------------
 * </pre>
 * @author Omistaja
 * @version 18.3.2025
 *
 */
public class Reseptit {

    private Ohjeet ohjeet = new Ohjeet();
    private Ainesosat ainesosat = new Ainesosat();
   
    

    /**
     * Listään uusi ainesosa resepteihin
     * @param ain lisättävä ainesosa
     */
    public void lisaaA(Ainesosa ain) {
        ainesosat.lisaa(ain);
    }

    
    /**
     * Palautaa ohjeiden määrän
     * @return ohjeiden määrä
     */
    public int getLkm() {
        return ohjeet.getLkm();
    }
    
    
    /**
     * Getteri ohjeen nimelle
     * @param ohje ohje, jonka nimi halutaan palauttaa
     * @return ohjeen nimi
     */
    public String getNimi(Ohje ohje) {
        return ohje.getNimi();
    }
    
    
    /**
     * palauttaa ohjeen tunnusnumeron
     * @param ohje ohje jonka tunnusnumero halutaan
     * @return ohjeen tunnusnumero
     */
    public int getId(Ohje ohje) {
        return ohje.getId();
    }
    
    
    /**
     * Getteri ohjeen tekstille
     * @param ohje ohje, jonka teksti halutaan palauttaa
     * @return ohjeen tekstiosa
     */
    public String getKesto(Ohje ohje) {
        return ohje.getKesto();
    }
    
    
    /**
     * Getteri ohjeen annoskoolle
     * @param ohje ohje, jonka annoskoko halutaan palauttaa
     * @return ohjeen annoskoko
     */
    public int getAnnoskoko(Ohje ohje) {
        return ohje.getAnnoskoko();
    }
    
    
    /**
     * Getteri ohjeen annoksen tyypille
     * @param ohje ohje, jonka annostyyppi halutaan palauttaa
     * @return ohjeen annostyyppi
     */
    public String getAnnostyyppi(Ohje ohje) {
        return ohje.getAnnostyyppi();
    }
    
    
    /**
     * Getteri ohjeen tekstille
     * @param ohje ohje, jonka teksti halutaan palauttaa
     * @return ohjeen tekstiosa
     */
    public String getTeksti(Ohje ohje) {
        return ohje.getTeksti();
    }
    
    
    /**
     * Palautaa ainesosien määrän
     * @return ainesosien määrä
     */
    public int getALkm() {
        return ainesosat.getALkm();
    }
    
    
    /**
     * @param ain kysyttävä ainesosa
     * @return palauttaa ainesosan nimen
     */
    public String getAnimi(Ainesosa ain) {
        return ain.getNimi();
    }
    
    
    /**
     * @param ain kysyttävä ainesosa
     * @return määrä
     */
    public double getAmaara(Ainesosa ain) {
        return ain.getMaara();
    }
    
    
    /**
     * @param ain kysyttävä ainesosa
     * @return yksikkö
     */
    public String getAyksikko(Ainesosa ain) {
        return ain.getYksikko();
    }


    /**
     * Poistaa ohjeista ne joilla on nro.
     * @param ohje poistettava ohje
     * @return montako ohjetta poistettiin
     */
    public int poista(Ohje ohje) {
        if ( ohje == null ) return 0;
        int ret = ohjeet.poista(ohje.getId()); 
        ainesosat.poistaOhjeenAinesosat(ohje.getId()); 
        return ret;
    }
    
    
    /**
     * Poistaa pelkän ohjeen eikä siihen liittyvää ainesosaluetteloa
     * @param ohje poistettava ohje
     * @return monta ohjetta poistettiin
     */
    public int poistaVainOhje(Ohje ohje) {
        if (ohje == null) return 0;
        int ret = ohjeet.poista(ohje.getId());
        return ret;
    }
    
    
    /** 
     * Poistaa tämän ainesosan 
     * @param ainesosa poistettava ainesosa
     */ 
    public void poistaAinesosa(Ainesosa ainesosa) { 
        ainesosat.poista(ainesosa); 
    } 


    /**
     * Lisää resepteihin uuden ohjeen
     * @param ohje lisättävä ohje
     * @example
     * <pre name="test">
     * Reseptit reseptit = new Reseptit();
     * Ohjeet ohjeet = new Ohjeet();
     * Ohje makaronilaatikko1 = new Ohje(), makaronilaatikko2 = new Ohje();
     * ohjeet.lisaa(makaronilaatikko1); 
     * ohjeet.lisaa(makaronilaatikko2); 
     * Collection<Ohje> loytyneet = reseptit.etsi("",-1); 
     * Iterator<Ohje> it = loytyneet.iterator();
     * it.next() === makaronilaatikko1;
     * it.next() === makaronilaatikko2;
     * it.next() === makaronilaatikko1;
     * </pre>
     */
    public void lisaa(Ohje ohje) {
        ohjeet.lisaa(ohje);
    }


    /** 
     * Palauttaa "taulukossa" hakuehtoon vastaavien ohjeitten viitteet 
     * @param hakuehto hakuehto  
     * @param k etsittävän kentän indeksi  
     * @return tietorakenteen löytyneistä ohjeista 
     */ 
    public Collection<Ohje> etsi(String hakuehto, int k) { 
        return ohjeet.etsi(hakuehto, k); 
    } 


    /**
     * Haetaan kaikki ohjeen ainesosat
     * @param ohje ohje jolle ainesosat haetaan
     * @return tietorakenne jossa viiteet löydetteyihin ainesosiin
     * @example
     * <pre name="test">
     * #THROWS SailoException
     * #import java.util.*;
     * 
     *  Reseptit reseptit = new Reseptit();
     *  Ohje makaroonilaatikko1 = new Ohje(), makaroonilaatikko2 = new Ohje();
     *  makaroonilaatikko1.valitseId(); makaroonilaatikko2.valitseId();
     *  int id1 = makaroonilaatikko1.getId();
     *  int id2 = makaroonilaatikko2.getId();
     *  Ainesosa maito11 = new Ainesosa(id1); reseptit.lisaaA(maito11);
     *  Ainesosa maito12 = new Ainesosa(id1); reseptit.lisaaA(maito12);
     *  Ainesosa maito21 = new Ainesosa(id2); reseptit.lisaaA(maito21);
     *  Ainesosa maito22 = new Ainesosa(id2); reseptit.lisaaA(maito22);
     *  Ainesosa maito23 = new Ainesosa(id2); reseptit.lisaaA(maito23);
     *  
     *  List<Ainesosa> loytyneet = new ArrayList<Ainesosa>();
     *  loytyneet.size() === 0; 
     *  loytyneet = reseptit.annaAinesosat(makaroonilaatikko1);
     *  loytyneet.size() === 2; 
     *  loytyneet.get(0) == maito11 === true;
     *  loytyneet.get(1) == maito12 === true;
     *  loytyneet = reseptit.annaAinesosat(makaroonilaatikko2);
     *  loytyneet.size() === 3; 
     *  loytyneet.get(0) == maito21 === true;
     * </pre> 
     */
    public List<Ainesosa> annaAinesosat(Ohje ohje) {
        return ainesosat.annaAinesosat(ohje.getId()); 
    }
    
    
    /**
     * @param i ohjeen indeksi
     * @return palauttaa indeksissä olevan ohjeen
     */
    public Ohje annaOhje(int i) {
        return ohjeet.anna(i);
    }
    
    
    /**
     * @param ohje korvattava ohje
     */
    public void korvaa(Ohje ohje) {
        ohjeet.korvaa(ohje);
    }
    
    
    /**
     * Asettaa tiedostojen perusnimet
     * @param nimi uusi nimi
     */
    public void setTied(String nimi) {
        File dir = new File(nimi);
        dir.mkdirs();
        String hakemistonNimi = "";
        if ( !nimi.isEmpty() ) hakemistonNimi = nimi +"/";
        ohjeet.setTiedPerusNimi(hakemistonNimi + "ohjeet");
        ainesosat.setTiedPerusNimi(hakemistonNimi + "ainesosat");
    }
    
    
    /**
     * @return palauttaaa ainesosat
     */
    public List<Ainesosa> TestiAinesosat1(){
        return ainesosat.MalliAinesosat1();
    }
    
    
    /**
     * @return palauttaaa ainesosat
     */
    public List<Ainesosa> TestiAinesosat2(){
        return ainesosat.MalliAinesosat2();
    }
    
    
    /**
     * Lukee ohjeen tiedot tiedostosta
     * @param nimi jota käyteään lukemisessa
     * @throws SailoException jos lukeminen epäonnistuu
     * @example
     * <pre name="test">
     * #THROWS SailoException 
     * #import java.io.*;
     * #import java.util.*;
     * 
     *  Reseptit reseptit = new Reseptit();
     *  Ohjeet ohjeet = new Ohjeet();
     *  
     *  Ohje makaronilaatikko1 = new Ohje(); makaronilaatikko1.Malliohje(); ohjeet.lisaa(makaronilaatikko1);
     *  Ohje makaronilaatikko2 = new Ohje(); makaronilaatikko2.Malliohje2(); ohjeet.lisaa(makaronilaatikko2);
     *  Ainesosa maito21 = new Ainesosa(); maito21.vastaaMaito(makaronilaatikko2.getId());
     *  Ainesosa maito11 = new Ainesosa(); maito11.vastaaMaito(makaronilaatikko1.getId());
     *  Ainesosa maito22 = new Ainesosa(); maito22.vastaaMaito(makaronilaatikko2.getId()); 
     *  Ainesosa maito12 = new Ainesosa(); maito12.vastaaMaito(makaronilaatikko1.getId()); 
     *  Ainesosa maito23 = new Ainesosa(); maito23.vastaaMaito(makaronilaatikko2.getId());
     *   
     *  String hakemisto = "testireseptit";
     *  File dir = new File(hakemisto);
     *  File ftied  = new File(hakemisto+"/ohjeet.dat");
     *  File fhtied = new File(hakemisto+"/ainesosat.dat");
     *  dir.mkdir();  
     *  ftied.delete();
     *  fhtied.delete();
     *  reseptit.lueTiedostosta(hakemisto); #THROWS SailoException
     *  reseptit.lisaa(makaronilaatikko1);
     *  reseptit.lisaa(makaronilaatikko2);
     *  reseptit.lisaaA(maito21);
     *  reseptit.lisaaA(maito11);
     *  reseptit.lisaaA(maito22);
     *  reseptit.lisaaA(maito12);
     *  reseptit.lisaaA(maito23);
     *  reseptit.talleta();
     *  reseptit = new Reseptit();
     *  reseptit.lueTiedostosta(hakemisto);
     *  Collection<Ohje> kaikki = reseptit.etsi("",-1); 
     *  Iterator<Ohje> it = kaikki.iterator();
     *  it.next() === makaronilaatikko1;
     *  it.next() === makaronilaatikko2;
     *  it.hasNext() === false;
     *  List<Ainesosa> loytyneet = reseptit.annaAinesosat(makaronilaatikko1);
     *  Iterator<Ainesosa> ih = loytyneet.iterator();
     *  ih.next() === maito11;
     *  ih.next() === maito12;
     *  ih.hasNext() === false;
     *  loytyneet = reseptit.annaAinesosat(makaronilaatikko2);
     *  ih = loytyneet.iterator();
     *  ih.next() === maito21;
     *  ih.next() === maito22;
     *  ih.next() === maito23;
     *  ih.hasNext() === false;
     *  reseptit.lisaa(makaronilaatikko2);
     *  reseptit.lisaaA(maito23);
     *  reseptit.talleta();
     *  ftied.delete()  === true;
     *  fhtied.delete() === true;
     *  File fbak = new File(hakemisto+"/ohjeet.bak");
     *  File fhbak = new File(hakemisto+"/ainesosat.bak");
     *  fbak.delete() === true;
     *  fhbak.delete() === true;
     *  dir.delete() === true;
     * </pre>
     */
    public void lueTiedostosta(String nimi) throws SailoException {
        ohjeet = new Ohjeet(); // jos luetaan olemassa olevaan niin helpoin tyhjentää näin
        ainesosat = new Ainesosat();

        setTied(nimi);
        ohjeet.lueTiedostosta();
        ainesosat.lueTiedostosta();
    }


    /**
     * Tallettaa ohjeen tiedot tiedostoon
     * Vaikka ohjeitten tallettamien epäonistuisi, niin yritetään silti tallettaa
     * ainesosia ennen poikkeuksen heittämistä.
     * @throws SailoException jos tallettamisessa ongelmia
     */
    public void talleta() throws SailoException {
        String virhe ="";
        try {
            ohjeet.tallenna();
        } catch (SailoException ex){
            virhe = ex.getMessage();
        }
        try {
            ainesosat.talleta();
        } catch ( SailoException ex ) {
            virhe += ex.getMessage();
        }
        if ( !"".equals(virhe) ) throw new SailoException(virhe);

    }

    
    /**
     * Testiohjelma Reseptille
     * @param args ei käytössä
     * 
     */
    public static void main(String[] args) {
        Reseptit reseptit = new Reseptit();

        Ohje makaronilaatikko1 = new Ohje(), makaronilaatikko2 = new Ohje();
        
        reseptit.lisaa(makaronilaatikko1);
        reseptit.lisaa(makaronilaatikko2);
        int id1 = makaronilaatikko1.getId();
        int id2 = makaronilaatikko2.getId();

        
        Ainesosa Maito11 = new Ainesosa(id1);
        Maito11.vastaaMaito(id1);
        reseptit.lisaaA(Maito11);
        Ainesosa Maito12 = new Ainesosa(id1);
        Maito12.vastaaMaito(id1);
        reseptit.lisaaA(Maito12);
        Ainesosa Maito21 = new Ainesosa(id2);
        Maito21.vastaaMaito(id2);
        reseptit.lisaaA(Maito21);
        Ainesosa Maito22 = new Ainesosa(id2);
        Maito22.vastaaMaito(id2);
        reseptit.lisaaA(Maito22);
        Ainesosa Maito23 = new Ainesosa(id2);
        Maito23.vastaaMaito(id2);
        reseptit.lisaaA(Maito23);


        System.out.println("============= Reseptit testi =================");
        System.out.println(makaronilaatikko1);
        System.out.println(makaronilaatikko2);
        Collection<Ohje> ohjeet = reseptit.etsi("", -1);
        int i = 0;
        for (Ohje ohje: ohjeet) {
            System.out.println("Ohje kohdassa: " + i);
            ohje.tulosta(System.out);
            List<Ainesosa> loytyneet = reseptit.annaAinesosat(ohje);
            for (Ainesosa ainesosa : loytyneet)
                ainesosa.tulosta(System.out);
            i++;
        }

    
    }

}
